#=======================================================================
#	$Id: Multi.pm,v 1.2 2005/09/09 11:11:31 pythontech Exp $
#	Treat multiple backends as a single entity.
#	Copyright (C) 2004,2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#-----------------------------------------------------------------------
#	Updates go in the first one.
#=======================================================================
package PythonTech::Triples::Multi;
use strict;
use PythonTech::Triples;

our @ISA = qw(PythonTech::Triples);

sub new {
    my($class,$props) = @_;
    my @dbs = @{$props->{'dbs'}};
    my $self = {
	dbs => [@dbs],
	main => $dbs[0],
    };
    bless $self, $class;
    return $self;
}

sub configure {
    my($self, $props) = shift;
    while (my($name,$val) = each %$props) {
	if ($name eq 'dbs') {
	    push @{$self->{'dbs'}}, @$val;
	} else {
	    die "$self: Unknown configuration option `$name'\n";
	}
    }
}

#-----------------------------------------------------------------------
#	Add a statement in the (first) database
#-----------------------------------------------------------------------
sub add {
    my($self, $subj,$pred,$obj) = @_;
    $self->{'main'}->add($subj,$pred,$obj);
}

#-----------------------------------------------------------------------
#	Remove a statement from the (first) database
#-----------------------------------------------------------------------
sub remove {
    my($self, $subj,$pred,$obj) = @_;
    $self->{'main'}->remove($subj,$pred,$obj);
}

#-----------------------------------------------------------------------
#	Generic query
#-----------------------------------------------------------------------
sub query {
    my($self, $subj,$pred,$obj) = @_;
    my @res;			# List of lists
    foreach my $db (@{$self->{'dbs'}}) {
	push @res, $db->query($subj,$pred,$obj);
    }
    return wantarray ? @res : $res[0];
}

sub query3 {
    my($self, $subj,$pred,$obj) = @_;
    my @res;			# List of lists
    foreach my $db (@{$self->{'dbs'}}) {
	push @res, $db->query3($subj,$pred,$obj);
    }
    return wantarray ? @res : $res[0];
}

#-----------------------------------------------------------------------
#	Optimised queries
#-----------------------------------------------------------------------
sub sp2o {
    my($self, $subj,$pred) = @_;
    my @res;
    foreach my $db (@{$self->{'dbs'}}) {
	push @res, $db->sp2o($subj,$pred);
    }
    return wantarray ? @res : $res[0];
}

#-----------------------------------------------------------------------
#	Commit any pending changes
#-----------------------------------------------------------------------
sub commit {
    my($self) = @_;
    $self->{'main'}->commit;
}

#-----------------------------------------------------------------------
#	Cancel any pending changes
#-----------------------------------------------------------------------
sub rollback {
    my($self) = @_;
    $self->{'main'}->rollback;
}

#-----------------------------------------------------------------------
#	Disconnect from / reconnect to backend
#-----------------------------------------------------------------------
sub disconnect {
    my($self) = @_;
    $self->{'main'}->disconnect;
}

sub reconnect {
    my($self, @args) = @_;
    $self->{'main'}->reconnect(@args);
}

1;
